@@ -1,5 +1,5 @@
 /*
- * Copyright (C)2011-2023 D. R. Commander.  All Rights Reserved.
+ * Copyright (C)2011-2024 D. R. Commander.  All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -91,7 +91,9 @@
   cArray = NULL; \
 }
 
-/* TurboJPEG 1.2.x: TJ::bufSize() */
+#define PAD(v, p)  ((v + (p) - 1) & (~((p) - 1)))
+
+/* TurboJPEG 1.2.x: TJ.bufSize() */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJ_bufSize
   (JNIEnv *env, jclass cls, jint width, jint height, jint jpegSubsamp)
 {
@@ -105,7 +107,7 @@ bailout:
   return (jint)retval;
 }
 
-/* TurboJPEG 1.4.x: TJ::bufSizeYUV() */
+/* TurboJPEG 1.4.x: TJ.bufSizeYUV() */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJ_bufSizeYUV__IIII
   (JNIEnv *env, jclass cls, jint width, jint align, jint height, jint subsamp)
 {
@@ -119,7 +121,7 @@ bailout:
   return (jint)retval;
 }
 
-/* TurboJPEG 1.4.x: TJ::planeSizeYUV() */
+/* TurboJPEG 1.4.x: TJ.planeSizeYUV() */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJ_planeSizeYUV__IIIII
   (JNIEnv *env, jclass cls, jint componentID, jint width, jint stride,
    jint height, jint subsamp)
@@ -134,7 +136,7 @@ bailout:
   return (jint)retval;
 }
 
-/* TurboJPEG 1.4.x: TJ::planeWidth() */
+/* TurboJPEG 1.4.x: TJ.planeWidth() */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJ_planeWidth__III
   (JNIEnv *env, jclass cls, jint componentID, jint width, jint subsamp)
 {
@@ -146,7 +148,7 @@ bailout:
   return retval;
 }
 
-/* TurboJPEG 1.4.x: TJ::planeHeight() */
+/* TurboJPEG 1.4.x: TJ.planeHeight() */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJ_planeHeight__III
   (JNIEnv *env, jclass cls, jint componentID, jint height, jint subsamp)
 {
@@ -158,7 +160,7 @@ bailout:
   return retval;
 }
 
-/* TurboJPEG 1.2.x: TJCompressor::init() */
+/* TurboJPEG 1.2.x: TJCompressor.init() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_init
   (JNIEnv *env, jobject obj)
 {
@@ -177,7 +179,7 @@ bailout:
   return;
 }
 
-/* TurboJPEG 3: TJCompressor::set() */
+/* TurboJPEG 3: TJCompressor.set() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_set
   (JNIEnv *env, jobject obj, jint param, jint value)
 {
@@ -192,7 +194,7 @@ bailout:
   return;
 }
 
-/* TurboJPEG 3: TJCompressor::get() */
+/* TurboJPEG 3: TJCompressor.get() */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_get
   (JNIEnv *env, jobject obj, jint param)
 {
@@ -277,7 +279,7 @@ bailout:
   return (jint)jpegSize;
 }
 
-/* TurboJPEG 3: TJCompressor::compress8() byte source */
+/* TurboJPEG 3: TJCompressor.compress8() byte source */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_compress8___3BIIIIII_3B
   (JNIEnv *env, jobject obj, jbyteArray src, jint x, jint y, jint width,
    jint pitch, jint height, jint pf, jbyteArray dst)
@@ -286,7 +288,7 @@ JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_compress8___
                                pf, dst);
 }
 
-/* TurboJPEG 3: TJCompressor::compress12() */
+/* TurboJPEG 3: TJCompressor.compress12() */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_compress12
   (JNIEnv *env, jobject obj, jshortArray src, jint x, jint y, jint width,
    jint pitch, jint height, jint pf, jbyteArray dst)
@@ -295,7 +297,7 @@ JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_compress12
                                height, pf, dst);
 }
 
-/* TurboJPEG 3: TJCompressor::compress16() */
+/* TurboJPEG 3: TJCompressor.compress16() */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_compress16
   (JNIEnv *env, jobject obj, jshortArray src, jint x, jint y, jint width,
    jint pitch, jint height, jint pf, jbyteArray dst)
@@ -304,7 +306,7 @@ JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_compress16
                                height, pf, dst);
 }
 
-/* TurboJPEG 3: TJCompressor::compress8() int source */
+/* TurboJPEG 3: TJCompressor.compress8() int source */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_compress8___3IIIIIII_3B
   (JNIEnv *env, jobject obj, jintArray src, jint x, jint y, jint width,
    jint stride, jint height, jint pf, jbyteArray dst)
@@ -321,7 +323,7 @@ bailout:
   return 0;
 }
 
-/* TurboJPEG 3: TJCompressor::compressFromYUV8() */
+/* TurboJPEG 3: TJCompressor.compressFromYUV8() */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_compressFromYUV8
   (JNIEnv *env, jobject obj, jobjectArray srcobjs, jintArray jSrcOffsets,
    jint width, jintArray jSrcStrides, jint height, jbyteArray dst)
@@ -502,7 +504,7 @@ bailout:
     SAFE_RELEASE(jDstPlanes[i], dstPlanesTmp[i]);
 }
 
-/* TurboJPEG 3: TJCompressor::encodeYUV8() byte source */
+/* TurboJPEG 3: TJCompressor.encodeYUV8() byte source */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_encodeYUV8___3BIIIIII_3_3B_3I_3I
   (JNIEnv *env, jobject obj, jbyteArray src, jint x, jint y, jint width,
    jint pitch, jint height, jint pf, jobjectArray dstobjs,
@@ -512,7 +514,7 @@ JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_encodeYUV8__
                           dstobjs, jDstOffsets, jDstStrides);
 }
 
-/* TurboJPEG 3: TJCompressor::encodeYUV8() int source */
+/* TurboJPEG 3: TJCompressor.encodeYUV8() int source */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_encodeYUV8___3IIIIIII_3_3B_3I_3I
   (JNIEnv *env, jobject obj, jintArray src, jint x, jint y, jint width,
    jint stride, jint height, jint pf, jobjectArray dstobjs,
@@ -531,7 +533,7 @@ bailout:
   return;
 }
 
-/* TurboJPEG 1.2.x: TJCompressor::destroy() */
+/* TurboJPEG 1.2.x: TJCompressor.destroy() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_destroy
   (JNIEnv *env, jobject obj)
 {
@@ -546,7 +548,7 @@ bailout:
   return;
 }
 
-/* TurboJPEG 1.2.x: TJDecompressor::init() */
+/* TurboJPEG 1.2.x: TJDecompressor.init() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_init
   (JNIEnv *env, jobject obj)
 {
@@ -565,21 +567,21 @@ bailout:
   return;
 }
 
-/* TurboJPEG 3: TJDecompressor::set() */
+/* TurboJPEG 3: TJDecompressor.set() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_set
   (JNIEnv *env, jobject obj, jint param, jint value)
 {
   Java_org_libjpegturbo_turbojpeg_TJCompressor_set(env, obj, param, value);
 }
 
-/* TurboJPEG 3: TJDecompressor::get() */
+/* TurboJPEG 3: TJDecompressor.get() */
 JNIEXPORT jint JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_get
   (JNIEnv *env, jobject obj, jint param)
 {
   return Java_org_libjpegturbo_turbojpeg_TJCompressor_get(env, obj, param);
 }
 
-/* TurboJPEG 1.2.x: TJDecompressor::getScalingFactors() */
+/* TurboJPEG 1.2.x: TJDecompressor.getScalingFactors() */
 JNIEXPORT jobjectArray JNICALL Java_org_libjpegturbo_turbojpeg_TJ_getScalingFactors
   (JNIEnv *env, jclass cls)
 {
@@ -610,7 +612,7 @@ bailout:
   return sfjava;
 }
 
-/* TurboJPEG 1.2.x: TJDecompressor::decompressHeader() */
+/* TurboJPEG 1.2.x: TJDecompressor.decompressHeader() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decompressHeader
   (JNIEnv *env, jobject obj, jbyteArray src, jint jpegSize)
 {
@@ -633,7 +635,7 @@ bailout:
   SAFE_RELEASE(src, jpegBuf);
 }
 
-/* TurboJPEG 3: TJDecompressor::setCroppingRegion() */
+/* TurboJPEG 3: TJDecompressor.setCroppingRegion() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_setCroppingRegion
   (JNIEnv *env, jobject obj)
 {
@@ -783,7 +785,7 @@ bailout:
   SAFE_RELEASE(src, jpegBuf);
 }
 
-/* TurboJPEG 3: TJDecompressor::decompress8() byte destination */
+/* TurboJPEG 3: TJDecompressor.decompress8() byte destination */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decompress8___3BI_3BIIII
   (JNIEnv *env, jobject obj, jbyteArray src, jint jpegSize, jbyteArray dst,
    jint x, jint y, jint pitch, jint pf)
@@ -792,7 +794,7 @@ JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decompress
                             pf);
 }
 
-/* TurboJPEG 3: TJDecompressor::decompress12() */
+/* TurboJPEG 3: TJDecompressor.decompress12() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decompress12
   (JNIEnv *env, jobject obj, jbyteArray src, jint jpegSize, jshortArray dst,
    jint x, jint y, jint pitch, jint pf)
@@ -801,7 +803,7 @@ JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decompress
                             pf);
 }
 
-/* TurboJPEG 3: TJDecompressor::decompress16() */
+/* TurboJPEG 3: TJDecompressor.decompress16() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decompress16
   (JNIEnv *env, jobject obj, jbyteArray src, jint jpegSize, jshortArray dst,
    jint x, jint y, jint pitch, jint pf)
@@ -810,7 +812,7 @@ JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decompress
                             pf);
 }
 
-/* TurboJPEG 3: TJDecompressor::decompress8() int destination */
+/* TurboJPEG 3: TJDecompressor.decompress8() int destination */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decompress8___3BI_3IIIII
   (JNIEnv *env, jobject obj, jbyteArray src, jint jpegSize, jintArray dst,
    jint x, jint y, jint stride, jint pf)
@@ -827,7 +829,7 @@ bailout:
   return;
 }
 
-/* TurboJPEG 3: TJDecompressor::decompressToYUV8() */
+/* TurboJPEG 3: TJDecompressor.decompressToYUV8() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decompressToYUV8
   (JNIEnv *env, jobject obj, jbyteArray src, jint jpegSize,
    jobjectArray dstobjs, jintArray jDstOffsets, jintArray jDstStrides)
@@ -1015,7 +1017,7 @@ bailout:
     SAFE_RELEASE(jSrcPlanes[i], srcPlanesTmp[i]);
 }
 
-/* TurboJPEG 3: TJDecompressor::decodeYUV8() byte destination */
+/* TurboJPEG 3: TJDecompressor.decodeYUV8() byte destination */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decodeYUV8___3_3B_3I_3I_3BIIIIII
   (JNIEnv *env, jobject obj, jobjectArray srcobjs, jintArray jSrcOffsets,
    jintArray jSrcStrides, jbyteArray dst, jint x, jint y, jint width,
@@ -1025,7 +1027,7 @@ JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decodeYUV8
                             1, x, y, width, pitch, height, pf);
 }
 
-/* TurboJPEG 3: TJDecompressor::decodeYUV8() int destination */
+/* TurboJPEG 3: TJDecompressor.decodeYUV8() int destination */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_decodeYUV8___3_3B_3I_3I_3IIIIIII
   (JNIEnv *env, jobject obj, jobjectArray srcobjs, jintArray jSrcOffsets,
    jintArray jSrcStrides, jintArray dst, jint x, jint y, jint width,
@@ -1044,7 +1046,7 @@ bailout:
   return;
 }
 
-/* TurboJPEG 1.2.x: TJTransformer::init() */
+/* TurboJPEG 1.2.x: TJTransformer.init() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJTransformer_init
   (JNIEnv *env, jobject obj)
 {
@@ -1128,7 +1130,7 @@ bailout:
   return -1;
 }
 
-/* TurboJPEG 1.2.x: TJTransformer::transform() */
+/* TurboJPEG 1.2.x: TJTransformer.transform() */
 JNIEXPORT jintArray JNICALL Java_org_libjpegturbo_turbojpeg_TJTransformer_transform
   (JNIEnv *env, jobject obj, jbyteArray jsrcBuf, jint jpegSize,
    jobjectArray dstobjs, jobjectArray tobjs)
@@ -1139,7 +1141,7 @@ JNIEXPORT jintArray JNICALL Java_org_libjpegturbo_turbojpeg_TJTransformer_transf
   size_t *dstSizes = NULL;
   tjtransform *t = NULL;
   jbyteArray *jdstBufs = NULL;
-  int i, jpegWidth = 0, jpegHeight = 0, jpegSubsamp;
+  int i, jpegWidth = 0, jpegHeight = 0, srcSubsamp;
   jintArray jdstSizes = 0;
   jint *dstSizesi = NULL;
   JNICustomFilterParams *params = NULL;
@@ -1152,8 +1154,7 @@ JNIEXPORT jintArray JNICALL Java_org_libjpegturbo_turbojpeg_TJTransformer_transf
     THROW_ARG("JPEG header has not yet been read");
   if ((jpegHeight = tj3Get(handle, TJPARAM_JPEGHEIGHT)) == -1)
     THROW_ARG("JPEG header has not yet been read");
-  if ((jpegSubsamp = tj3Get(handle, TJPARAM_SUBSAMP)) == TJSAMP_UNKNOWN)
-    THROW_ARG("TJPARAM_SUBSAMP must be specified");
+  srcSubsamp = tj3Get(handle, TJPARAM_SUBSAMP);
 
   n = (*env)->GetArrayLength(env, dstobjs);
   if (n != (*env)->GetArrayLength(env, tobjs))
@@ -1212,16 +1213,21 @@ JNIEXPORT jintArray JNICALL Java_org_libjpegturbo_turbojpeg_TJTransformer_transf
 
   for (i = 0; i < n; i++) {
     int w = jpegWidth, h = jpegHeight;
+    int dstSubsamp = (t[i].options & TJXOPT_GRAY) ? TJSAMP_GRAY : srcSubsamp;
 
     if (t[i].op == TJXOP_TRANSPOSE || t[i].op == TJXOP_TRANSVERSE ||
         t[i].op == TJXOP_ROT90 || t[i].op == TJXOP_ROT270) {
       w = jpegHeight;  h = jpegWidth;
+      if (dstSubsamp == TJSAMP_422) dstSubsamp = TJSAMP_440;
+      else if (dstSubsamp == TJSAMP_440) dstSubsamp = TJSAMP_422;
+      else if (dstSubsamp == TJSAMP_411) dstSubsamp = TJSAMP_441;
+      else if (dstSubsamp == TJSAMP_441) dstSubsamp = TJSAMP_411;
     }
     if (t[i].r.w != 0) w = t[i].r.w;
     if (t[i].r.h != 0) h = t[i].r.h;
     BAILIF0(jdstBufs[i] = (*env)->GetObjectArrayElement(env, dstobjs, i));
     if ((size_t)(*env)->GetArrayLength(env, jdstBufs[i]) <
-        tj3JPEGBufSize(w, h, jpegSubsamp))
+        tj3JPEGBufSize(w, h, dstSubsamp))
       THROW_ARG("Destination buffer is not large enough");
   }
   BAILIF0NOEC(jpegBuf = (*env)->GetPrimitiveArrayCritical(env, jsrcBuf, 0));
@@ -1260,7 +1266,7 @@ bailout:
   return jdstSizes;
 }
 
-/* TurboJPEG 1.2.x: TJDecompressor::destroy() */
+/* TurboJPEG 1.2.x: TJDecompressor.destroy() */
 JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_destroy
   (JNIEnv *env, jobject obj)
 {
@@ -1274,7 +1280,8 @@ JNIEXPORT jobject JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_loadImage
 {
   tjhandle handle = NULL;
   void *dstBuf = NULL, *jdstPtr;
-  int width, *warr, height, *harr, pixelFormat, *pfarr, n;
+  int width, *warr, height, *harr, pixelFormat, *pfarr;
+  jsize arraySize, pitch;
   const char *filename = NULL;
   jboolean isCopy;
   jobject jdstBuf = NULL;
@@ -1331,13 +1338,14 @@ JNIEXPORT jobject JNICALL Java_org_libjpegturbo_turbojpeg_TJCompressor_loadImage
   pfarr[0] = pixelFormat;
   (*env)->ReleasePrimitiveArrayCritical(env, jpixelFormat, pfarr, 0);
 
-  n = width * height * tjPixelSize[pixelFormat];
+  pitch = PAD(width * tjPixelSize[pixelFormat], align);
+  arraySize = pitch * height;
   if (precision == 8)
-    jdstBuf = (*env)->NewByteArray(env, n);
+    jdstBuf = (*env)->NewByteArray(env, arraySize);
   else
-    jdstBuf = (*env)->NewShortArray(env, n);
+    jdstBuf = (*env)->NewShortArray(env, arraySize);
   BAILIF0NOEC(jdstPtr = (*env)->GetPrimitiveArrayCritical(env, jdstBuf, 0));
-  memcpy(jdstPtr, dstBuf, n * (precision > 8 ? 2 : 1));
+  memcpy(jdstPtr, dstBuf, arraySize * (precision > 8 ? 2 : 1));
   (*env)->ReleasePrimitiveArrayCritical(env, jdstBuf, jdstPtr, 0);
 
 bailout:
@@ -1354,29 +1362,33 @@ JNIEXPORT void JNICALL Java_org_libjpegturbo_turbojpeg_TJDecompressor_saveImage
   tjhandle handle = NULL;
   void *srcBuf = NULL, *jsrcPtr;
   const char *filename = NULL;
-  int n;
+  jsize arraySize, actualPitch;
   jboolean isCopy;
 
   GET_HANDLE();
 
   if ((precision != 8 && precision != 12 && precision != 16) ||
-      jfilename == NULL || jsrcBuf == NULL || width < 1 || height < 1 ||
-      pixelFormat < 0 || pixelFormat >= TJ_NUMPF)
+      jfilename == NULL || jsrcBuf == NULL || width < 1 || pitch < 0 ||
+      height < 1 || pixelFormat < 0 ||
+      pixelFormat >= org_libjpegturbo_turbojpeg_TJ_NUMPF)
     THROW_ARG("Invalid argument in saveImage()");
+  if (org_libjpegturbo_turbojpeg_TJ_NUMPF != TJ_NUMPF)
+    THROW_ARG("Mismatch between Java and C API");
 
   if ((unsigned long long)width * (unsigned long long)height *
       (unsigned long long)tjPixelSize[pixelFormat] >
       (unsigned long long)((unsigned int)-1))
     THROW_ARG("Image is too large");
-  n = width * height * tjPixelSize[pixelFormat];
-  if ((*env)->GetArrayLength(env, jsrcBuf) < n)
+  actualPitch = (pitch == 0) ? width * tjPixelSize[pixelFormat] : pitch;
+  arraySize = actualPitch * height;
+  if ((*env)->GetArrayLength(env, jsrcBuf) < arraySize)
     THROW_ARG("Source buffer is not large enough");
 
-  if ((srcBuf = malloc(n * (precision > 8 ? 2 : 1))) == NULL)
+  if ((srcBuf = malloc(arraySize * (precision > 8 ? 2 : 1))) == NULL)
     THROW_MEM();
 
   BAILIF0NOEC(jsrcPtr = (*env)->GetPrimitiveArrayCritical(env, jsrcBuf, 0));
-  memcpy(srcBuf, jsrcPtr, n * (precision > 8 ? 2 : 1));
+  memcpy(srcBuf, jsrcPtr, arraySize * (precision > 8 ? 2 : 1));
   (*env)->ReleasePrimitiveArrayCritical(env, jsrcBuf, jsrcPtr, 0);
   BAILIF0(filename = (*env)->GetStringUTFChars(env, jfilename, &isCopy));
 
